@@ -25,6 +25,9 @@
 #'  \code{geojsonio::geojson_sp}. If \code{FALSE} and there are no attributes associated with
 #'  the geometries, a \code{GeometryCollection} will be output. Ignored for \code{Spatial}
 #'  objects, as the output is always the same as the input.
+#' @param sys Should the system mapshaper be used instead of the bundled mapshaper? Gives
+#'   better performance on large files. Requires the mapshapr node package to be installed
+#'   and on the PATH.
 #'
 #' @return clipped target in the same class as the input target
 #'
@@ -70,68 +73,68 @@
 #'
 #' @export
 ms_clip <- function(target, clip = NULL, bbox = NULL, remove_slivers = FALSE,
-                    force_FC = TRUE) {
+                    force_FC = TRUE, sys = FALSE) {
   if (!is.logical(force_FC)) stop("force_FC must be TRUE or FALSE")
   UseMethod("ms_clip")
 }
 
 #' @export
 ms_clip.character <- function(target, clip = NULL, bbox = NULL,
-                              remove_slivers = FALSE, force_FC = TRUE) {
+                              remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   target <- check_character_input(target)
 
   clip_erase_json(target = target, overlay_layer = clip, type = "clip",
-                  remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                  remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 
 }
 
 #' @export
 ms_clip.geo_json <- function(target, clip = NULL, bbox = NULL, remove_slivers = FALSE,
-                             force_FC = TRUE) {
+                             force_FC = TRUE, sys = FALSE) {
   clip_erase_json(target = target, overlay_layer = clip, type = "clip",
-                  remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                  remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_clip.geo_list <- function(target, clip = NULL, bbox = NULL, remove_slivers = FALSE,
-                             force_FC = TRUE) {
+                             force_FC = TRUE, sys = FALSE) {
   clip_erase_geo_list(target = target, overlay_layer = clip, type = "clip",
-                      remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                      remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_clip.SpatialPolygons <- function(target, clip = NULL, bbox = NULL,
-                                    remove_slivers = FALSE, force_FC = TRUE) {
+                                    remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sp(target = target, overlay_layer = clip, type = "clip",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_clip.SpatialLines <- function(target, clip = NULL, bbox = NULL,
-                                 remove_slivers = FALSE, force_FC = TRUE) {
+                                 remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sp(target = target, overlay_layer = clip, type = "clip",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_clip.SpatialPoints <- function(target, clip = NULL, bbox = NULL,
-                                  remove_slivers = FALSE, force_FC = TRUE) {
+                                  remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sp(target = target, overlay_layer = clip, type = "clip",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_clip.sf <- function(target, clip = NULL, bbox = NULL,
-                       remove_slivers = FALSE, force_FC = TRUE) {
+                       remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sf(target = target, overlay_layer = clip, type = "clip",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_clip.sfc <- function(target, clip = NULL, bbox = NULL,
-                       remove_slivers = FALSE, force_FC = TRUE) {
+                       remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sf(target = target, overlay_layer = clip, type = "clip",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' Remove features or portions of features that fall inside a specified area
@@ -159,6 +162,10 @@ ms_clip.sfc <- function(target, clip = NULL, bbox = NULL,
 #'  \code{geojsonio::geojson_sp}. If \code{FALSE} and there are no attributes associated with
 #'  the geometries, a \code{GeometryCollection} will be output. Ignored for \code{Spatial}
 #'  objects, as the output is always the same class as the input.
+#' @param sys Should the system mapshaper be used instead of the bundled mapshaper? Gives
+#'   better performance on large files. Requires the mapshapr node package to be installed
+#'   and on the PATH.
+#'
 #'@return erased target in the same format as the input target
 #' @examples
 #' library(geojsonio, quietly = TRUE)
@@ -206,72 +213,72 @@ ms_clip.sfc <- function(target, clip = NULL, bbox = NULL,
 #'
 #'@export
 ms_erase <- function(target, erase = NULL, bbox = NULL,
-                     remove_slivers = FALSE, force_FC = TRUE) {
+                     remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   if (!is.logical(force_FC)) stop("force_FC must be TRUE or FALSE")
   UseMethod("ms_erase")
 }
 
 #' @export
 ms_erase.character <- function(target, erase = NULL, bbox = NULL,
-                               remove_slivers = FALSE, force_FC = TRUE) {
+                               remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   target <- check_character_input(target)
 
   clip_erase_json(target = target, overlay_layer = erase, type = "erase",
-                  remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                  remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 
 }
 
 #' @export
 ms_erase.geo_json <- function(target, erase = NULL, bbox = NULL,
-                              remove_slivers = FALSE, force_FC = TRUE) {
+                              remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_json(target = target, overlay_layer = erase, type = "erase",
-                  remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                  remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_erase.geo_list <- function(target, erase = NULL, bbox = NULL,
-                              remove_slivers = FALSE, force_FC = TRUE) {
+                              remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_geo_list(target = target, overlay_layer = erase, type = "erase",
-                      remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                      remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_erase.SpatialPolygons <- function(target, erase = NULL, bbox = NULL,
-                                     remove_slivers = FALSE, force_FC = TRUE) {
+                                     remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sp(target = target, overlay_layer = erase, type = "erase",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_erase.SpatialLines <- function(target, erase = NULL, bbox = NULL,
-                                  remove_slivers = FALSE, force_FC = TRUE) {
+                                  remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sp(target = target, overlay_layer = erase, type = "erase",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_erase.SpatialPoints <- function(target, erase = NULL, bbox = NULL,
-                                   remove_slivers = FALSE, force_FC = TRUE) {
+                                   remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sp(target = target, overlay_layer = erase, type = "erase",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_erase.sf <- function(target, erase = NULL, bbox = NULL,
-                       remove_slivers = FALSE, force_FC = TRUE) {
+                       remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sf(target = target, overlay_layer = erase, type = "erase",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_erase.sfc <- function(target, erase = NULL, bbox = NULL,
-                        remove_slivers = FALSE, force_FC = TRUE) {
+                        remove_slivers = FALSE, force_FC = TRUE, sys = FALSE) {
   clip_erase_sf(target = target, overlay_layer = erase, type = "erase",
-                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 clip_erase_json <- function(target, overlay_layer, bbox, remove_slivers, type,
-                            force_FC) {
+                            force_FC, sys) {
 
   check_overlay_bbox(overlay_layer = overlay_layer, bbox = bbox, type = type)
 
@@ -280,25 +287,25 @@ clip_erase_json <- function(target, overlay_layer, bbox, remove_slivers, type,
   }
 
   mapshaper_clip_erase(target_layer = target, overlay_layer = overlay_layer, type = type,
-                       remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                       remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
 }
 
 clip_erase_geo_list <- function(target, overlay_layer, bbox, type,
-                                remove_slivers, force_FC) {
+                                remove_slivers, force_FC, sys) {
 
   check_overlay_bbox(overlay_layer = overlay_layer, bbox = bbox, type = type)
 
   if (is.null(bbox)) {
     if (!is(overlay_layer, "geo_list")) stop("both target and ", type, " must be class geo_list")
-    overlay_layer <- geojsonio::geojson_json(overlay_layer)
+    overlay_layer <- geo_list_to_json(overlay_layer)
   }
-  target <- geojsonio::geojson_json(target)
+  target <- geo_list_to_json(target)
   ret <- clip_erase_json(target = target, overlay_layer = overlay_layer, type = type,
-                         remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC)
+                         remove_slivers = remove_slivers, bbox = bbox, force_FC = force_FC, sys = sys)
   geojsonio::geojson_list(ret)
 }
 
-clip_erase_sp <- function(target, overlay_layer, bbox, type, remove_slivers, force_FC) {
+clip_erase_sp <- function(target, overlay_layer, bbox, type, remove_slivers, force_FC, sys) {
 
   check_overlay_bbox(overlay_layer = overlay_layer, bbox = bbox, type = type)
 
@@ -311,15 +318,15 @@ clip_erase_sp <- function(target, overlay_layer, bbox, type, remove_slivers, for
               type, " to target CRS")
       overlay_layer <- sp::spTransform(overlay_layer, target_proj)
     }
-    overlay_geojson <- sp_to_GeoJSON(overlay_layer)
+    overlay_geojson <- sp_to_GeoJSON(overlay_layer, file = sys)
   }
 
-  target_geojson <- sp_to_GeoJSON(target)
+  target_geojson <- sp_to_GeoJSON(target, file = sys)
 
   result <- mapshaper_clip_erase(target_layer = target_geojson,
                                  overlay_layer = overlay_geojson,
                                  type = type, remove_slivers = remove_slivers,
-                                 bbox = bbox, force_FC = TRUE)
+                                 bbox = bbox, force_FC = TRUE, sys = sys)
 
   ret <- GeoJSON_to_sp(result, target_proj)
 
@@ -331,9 +338,7 @@ clip_erase_sp <- function(target, overlay_layer, bbox, type, remove_slivers, for
   ret
 }
 
-clip_erase_sf <- function(target, overlay_layer, bbox, type, remove_slivers, force_FC) {
-
-  check_sf_pkg()
+clip_erase_sf <- function(target, overlay_layer, bbox, type, remove_slivers, force_FC, sys) {
 
   check_overlay_bbox(overlay_layer = overlay_layer, bbox = bbox, type = type)
 
@@ -354,15 +359,15 @@ clip_erase_sf <- function(target, overlay_layer, bbox, type, remove_slivers, for
               type, " to target CRS")
       overlay_layer <- sf::st_transform(overlay_layer, target_proj)
     }
-    overlay_geojson <- sf_to_GeoJSON(overlay_layer)
+    overlay_geojson <- sf_to_GeoJSON(overlay_layer, file = sys)
   }
 
-  target_geojson <- sf_to_GeoJSON(target)
+  target_geojson <- sf_to_GeoJSON(target, file = sys)
 
   result <- mapshaper_clip_erase(target_layer = target_geojson,
                                  overlay_layer = overlay_geojson,
                                  type = type, remove_slivers = remove_slivers,
-                                 bbox = bbox, force_FC = TRUE)
+                                 bbox = bbox, force_FC = TRUE, sys = sys)
 
   ret <- GeoJSON_to_sf(result, target_proj)
 
@@ -395,80 +400,54 @@ check_overlay_bbox <- function(overlay_layer, bbox, type) {
 }
 
 mapshaper_clip_erase <- function(target_layer, overlay_layer, bbox, type,
-                                 remove_slivers, force_FC) {
+                                 remove_slivers, force_FC, sys) {
 
   remove_slivers <- ifelse(remove_slivers, "remove-slivers", "")
 
   if (!is.null(bbox)) {
     cmd <- paste0("-", type, " bbox=",paste0(bbox, collapse = ","), " ",
                   remove_slivers)
-    out <- apply_mapshaper_commands(target_layer, cmd, force_FC = force_FC)
+    out <- apply_mapshaper_commands(target_layer, cmd, force_FC = force_FC, sys = sys)
   } else if (!is.null(overlay_layer)) {
 
-    ms <- ms_make_ctx()
-    ## Import the layers into the V8 session
-    ms$assign("target_geojson", target_layer)
-    ms$assign("overlay_geojson", overlay_layer)
+    if (sys) {
+      cmd <- paste0("-", type)
+      out <- sys_mapshaper(data = target_layer, data2 = overlay_layer, command = cmd)
+    } else {
 
-    ## convert geojson to mapshaper datasets, give each layer a name which can be
-    ## referred to in the commands as target and clipping layer
-    ## Then merge the datasets into one that can be passed on to runCommand
-    ms$eval('
-      var target_layer = mapshaper.internal.importFileContent(target_geojson, null, {});
-      target_layer.layers[0].name = "target_layer";
-      var overlay_layer = mapshaper.internal.importFileContent(overlay_geojson, null, {});
-      overlay_layer.layers[0].name = "overlay_layer";
-      var dataset = mapshaper.internal.mergeDatasets([target_layer, overlay_layer]);'
-    )
+      ms <- ms_make_ctx()
+      ## Import the layers into the V8 session
+      ms$assign("target_geojson", target_layer)
+      ms$assign("overlay_geojson", overlay_layer)
 
+      # Combine them into a single object
+      ms$eval("
+      var data_layers = {'target.json': target_geojson, 'overlay.json': overlay_geojson}
+    ")
 
-    ## Construct the command string; referring to layer names as assigned above
-    command <- paste0("-", type,
-                      " target=target_layer source=overlay_layer ",
-                      remove_slivers,
-                      " -o format=geojson")
+      ## Construct the command string; referring to layer names as assigned above
+      command <- paste0("-i overlay.json -i target.json -", type,
+                        " overlay ",
+                        remove_slivers)
 
-    ## Parse the commands
-    ms$eval(paste0('var command = mapshaper.internal.parseCommands("',
-                   command, '")'))
+      if (force_FC) {
+        command <- paste(command, add_dummy_id_command())
+      }
 
-    ## use runCommand to run the clipping function on the merged dataset and return
-    ## it to R
-    ms$eval(
-      "
-      var return_data = {};
-      mapshaper.runCommand(command[0], dataset, function(err,data){
-        // This chunk from Mapshaper.ProcessFileContent to get output options:
-        // if last command is -o, use -o options for exporting
-        outCmd = command[command.length-1];
-        if (outCmd && outCmd.name == 'o') {
-        outOpts = command.pop().options;
-        } else {
-        outOpts = {};
-        }
-        // Convert dataset to geojson for export
-        // (or if other format supplied in output options)
-        return_data = mapshaper.internal.exportFileContent(data, outOpts)[0].content;
-      })"
-    )
+      command <- paste(command, "-o format=geojson")
 
-    ## Add a dummy id to force to FeatureCollection
-    if (force_FC) {
-      add_id_cmd <- add_dummy_id_command()
-      ms$eval(paste0(
-        "
-      // make sure that a FeatureCollection is returned by applying a dummy id
-      // to each geometry. Otherwise if there are no attributes (i.e., just
-      // geometries, a GeometryCollection is output which readOGR doesn't like)
-      mapshaper.applyCommands(\"", add_id_cmd, "\", return_data, function(Error, data) {
-          if (Error) console.error(Error);
-          return_data = data;
-      })"
-      ))
+      # Create an object to hold the return value
+      ms$eval("var return_data;")
+
+      # Run the commands on the data
+      ms$eval(paste0('mapshaper.applyCommands("',
+                     command,
+                     '", data_layers, ',
+                     callback(), ')'))
+
+      out <- ms$get("return_data")[["target.json"]]
+      out <- class_geo_json(out)
     }
-
-    out <- ms$get("return_data")
-    out <- class_geo_json(out)
   }
   out
 }
