@@ -22,6 +22,9 @@
 #'  \code{geojsonio::geojson_sp}. If \code{FALSE} and there are no attributes associated with
 #'  the geometries, a \code{GeometryCollection} will be output. Ignored for \code{Spatial}
 #'  objects, as the output is always the same class as the input.
+#' @param sys Should the system mapshaper be used instead of the bundled mapshaper? Gives
+#'   better performance on large files. Requires the mapshapr node package to be installed
+#'   and on the PATH.
 #'
 #' @return the same class as the input
 #'
@@ -59,81 +62,81 @@
 #'
 #' @export
 ms_dissolve <- function(input, field = NULL, sum_fields = NULL, copy_fields = NULL,
-                        weight = NULL, snap = TRUE, force_FC = TRUE) {
+                        weight = NULL, snap = TRUE, force_FC = TRUE, sys = FALSE) {
   UseMethod("ms_dissolve")
 }
 
 #' @export
 ms_dissolve.character <- function(input, field = NULL, sum_fields = NULL, copy_fields = NULL,
-                                  weight = NULL, snap = TRUE, force_FC = TRUE) {
+                                  weight = NULL, snap = TRUE, force_FC = TRUE, sys = FALSE) {
   input <- check_character_input(input)
 
   call <- make_dissolve_call(field = field, sum_fields = sum_fields, weight = weight,
                              copy_fields = copy_fields, snap = snap)
 
-  apply_mapshaper_commands(data = input, command = call, force_FC = force_FC)
+  apply_mapshaper_commands(data = input, command = call, force_FC = force_FC, sys = sys)
 
 }
 
 #' @export
 ms_dissolve.geo_json <- function(input, field = NULL, sum_fields = NULL, copy_fields = NULL,
-                                 weight = NULL, snap = TRUE, force_FC = TRUE) {
+                                 weight = NULL, snap = TRUE, force_FC = TRUE, sys = FALSE) {
 
   call <- make_dissolve_call(field = field, sum_fields = sum_fields, weight = weight,
                              copy_fields = copy_fields, snap = snap)
 
-  apply_mapshaper_commands(data = input, command = call, force_FC = force_FC)
+  apply_mapshaper_commands(data = input, command = call, force_FC = force_FC, sys = sys)
 }
 
 #' @export
 ms_dissolve.geo_list <- function(input, field = NULL, sum_fields = NULL, copy_fields = NULL,
-                                 weight = NULL, snap = TRUE, force_FC = TRUE) {
+                                 weight = NULL, snap = TRUE, force_FC = TRUE, sys = FALSE) {
 
   call <- make_dissolve_call(field = field, sum_fields = sum_fields, weight = weight,
                              copy_fields = copy_fields, snap = snap)
 
-  geojson <- geojsonio::geojson_json(input)
+  geojson <- geo_list_to_json(input)
 
-  ret <- apply_mapshaper_commands(data = geojson, command = call, force_FC = force_FC)
+  ret <- apply_mapshaper_commands(data = geojson, command = call, force_FC = force_FC, sys = sys)
 
   geojsonio::geojson_list(ret)
 }
 
 #' @export
 ms_dissolve.SpatialPolygons <- function(input, field = NULL, sum_fields = NULL, copy_fields = NULL,
-                                        weight = NULL, snap = TRUE, force_FC = TRUE) {
+                                        weight = NULL, snap = TRUE, force_FC = TRUE, sys = FALSE) {
  dissolve_sp(input = input, field = field, sum_fields = sum_fields, copy_fields = copy_fields,
-             weight = weight, snap = snap)
+             weight = weight, snap = snap, sys = sys)
 }
 
 #' @export
 ms_dissolve.SpatialPoints <- function(input, field = NULL, sum_fields = NULL, copy_fields = NULL,
-                                      weight = NULL, snap = TRUE, force_FC = TRUE) {
+                                      weight = NULL, snap = TRUE, force_FC = TRUE, sys = FALSE) {
   dissolve_sp(input = input, field = field, sum_fields = sum_fields, copy_fields = copy_fields,
-              weight = weight, snap = snap)
+              weight = weight, snap = snap, sys = sys)
 }
 
 #' @export
 ms_dissolve.sf <- function(input, field = NULL, sum_fields = NULL, copy_fields = NULL,
-                           weight = NULL, snap = TRUE, force_FC = TRUE) {
+                           weight = NULL, snap = TRUE, force_FC = TRUE, sys = FALSE) {
   if (!is.null(weight) && !(weight %in% names(input))) {
     stop("specified 'weight' column not present in input data", call. = FALSE)
   }
 
   dissolve_sf(input = input, field = field, sum_fields = sum_fields, copy_fields = copy_fields,
-              weight = weight, snap = snap)
+              weight = weight, snap = snap, sys = sys)
 
 }
 
 #' @export
 ms_dissolve.sfc <- function(input, field = NULL, sum_fields = NULL, copy_fields = NULL,
-                            weight = NULL, snap = TRUE, force_FC = TRUE) {
+                            weight = NULL, snap = TRUE, force_FC = TRUE, sys = FALSE) {
   if (!is.null(weight)) {
     warning("'weight' cannot be used with sfc objects. Ignoring it and proceeding...")
   }
 
   dissolve_sf(input = input, field = field, sum_fields = sum_fields, copy_fields = copy_fields,
-              weight = NULL, snap = snap)
+              weight = NULL, snap = snap, sys = sys)
 }
 
 make_dissolve_call <- function(field, sum_fields, copy_fields, weight, snap) {
@@ -163,7 +166,7 @@ make_dissolve_call <- function(field, sum_fields, copy_fields, weight, snap) {
   call
 }
 
-dissolve_sp <- function(input, field, sum_fields, copy_fields, weight, snap) {
+dissolve_sp <- function(input, field, sum_fields, copy_fields, weight, snap, sys) {
 
   if (!inherits(input, "SpatialPointsDataFrame") && !is.null(weight)) {
     stop("weight arguments only applies to points with attributes", call. = FALSE)
@@ -176,10 +179,10 @@ dissolve_sp <- function(input, field, sum_fields, copy_fields, weight, snap) {
   call <- make_dissolve_call(field = field, sum_fields = sum_fields, copy_fields = copy_fields,
                              weight = weight, snap = snap)
 
-  ms_sp(input = input, call = call)
+  ms_sp(input = input, call = call, sys = sys)
 }
 
-dissolve_sf <- function(input, field, sum_fields, copy_fields, weight, snap) {
+dissolve_sf <- function(input, field, sum_fields, copy_fields, weight, snap, sys) {
 
   if (!all(sf::st_is(input, c("POINT", "MULTIPOINT", "POLYGON", "MULTIPOLYGON")))) {
     stop("ms_dissolve only works with (MULTI)POINT or (MULTI)POLYGON", call. = FALSE)
@@ -192,5 +195,5 @@ dissolve_sf <- function(input, field, sum_fields, copy_fields, weight, snap) {
   call <- make_dissolve_call(field = field, sum_fields = sum_fields, copy_fields = copy_fields,
                              weight = weight, snap = snap)
 
-  ms_sf(input = input, call = call)
+  ms_sf(input = input, call = call, sys = sys)
 }
