@@ -20,6 +20,9 @@
 #'   and \code{geojsonio::geojson_sp}. If \code{FALSE} and there are no
 #'   attributes associated with the geometries, a \code{GeometryCollection} will
 #'   be output. Ignored for \code{Spatial} objects.
+#' @param sys Should the system mapshaper be used instead of the bundled mapshaper? Gives
+#'   better performance on large files. Requires the mapshapr node package to be installed
+#'   and on the PATH.
 #'
 #' @return topological boundaries as lines, in the same class as the input
 #'
@@ -55,42 +58,45 @@
 #' plot(out)
 #'
 #' @export
-ms_lines <- function(input, fields = NULL, force_FC = TRUE) {
+ms_lines <- function(input, fields = NULL, force_FC = TRUE, sys = FALSE) {
   if (!is.null(fields) && !is.character(fields)) stop("fields must be a character vector of field names")
   if (!is.logical(force_FC)) stop("force_FC must be TRUE or FALSE")
   UseMethod("ms_lines")
 }
 
 #' @export
-ms_lines.character <- function(input, fields = NULL, force_FC = TRUE) {
+ms_lines.character <- function(input, fields = NULL, force_FC = TRUE, sys = FALSE) {
   input <- check_character_input(input)
 
   command <- make_lines_call(fields)
 
-  apply_mapshaper_commands(data = input, command = command, force_FC = force_FC)
+  apply_mapshaper_commands(data = input, command = command, force_FC = force_FC,
+                           sys = sys)
 
 }
 
 #' @export
-ms_lines.geo_json <- function(input, fields = NULL, force_FC = TRUE) {
+ms_lines.geo_json <- function(input, fields = NULL, force_FC = TRUE, sys = FALSE) {
   command <- make_lines_call(fields)
 
-  apply_mapshaper_commands(data = input, command = command, force_FC = force_FC)
+  apply_mapshaper_commands(data = input, command = command, force_FC = force_FC,
+                           sys = sys)
 }
 
 #' @export
-ms_lines.geo_list <- function(input, fields = NULL, force_FC = TRUE) {
-  geojson <- geojsonio::geojson_json(input)
+ms_lines.geo_list <- function(input, fields = NULL, force_FC = TRUE, sys = FALSE) {
+  geojson <- geo_list_to_json(input)
 
   command <- make_lines_call(fields)
 
-  ret <- apply_mapshaper_commands(data = geojson, command = command, force_FC = force_FC)
+  ret <- apply_mapshaper_commands(data = geojson, command = command,
+                                  force_FC = force_FC, sys = sys)
 
   geojsonio::geojson_list(ret)
 }
 
 #' @export
-ms_lines.SpatialPolygons <- function(input, fields = NULL, force_FC) {
+ms_lines.SpatialPolygons <- function(input, fields = NULL, force_FC, sys = FALSE) {
 
   if (.hasSlot(input, "data")) {
     if (!all(fields %in% names(input@data))) {
@@ -100,37 +106,37 @@ ms_lines.SpatialPolygons <- function(input, fields = NULL, force_FC) {
 
   command <- make_lines_call(fields)
 
-  ms_sp(input, command)
+  ms_sp(input, command, sys = sys)
 }
 
 #' @export
-ms_lines.sf <- function(input, fields = NULL, force_FC) {
+ms_lines.sf <- function(input, fields = NULL, force_FC, sys = FALSE) {
 
   if (!all(fields %in% names(input))) {
     stop("not all fields specified exist in input data")
   }
 
-  lines_sf(input = input, fields = fields)
+  lines_sf(input = input, fields = fields, sys = sys)
 }
 
 #' @export
-ms_lines.sfc <- function(input, fields = NULL, force_FC) {
+ms_lines.sfc <- function(input, fields = NULL, force_FC, sys = FALSE) {
 
   if (!is.null(fields)) {
     stop("Do not specify fields for sfc classes", call. = FALSE)
   }
 
-  lines_sf(input = input, fields = fields)
+  lines_sf(input = input, fields = fields, sys = sys)
 }
 
-lines_sf <- function(input, fields) {
+lines_sf <- function(input, fields, sys) {
   if (!all(sf::st_is(input, c("POLYGON", "MULTIPOLYGON")))) {
     stop("ms_lines only works with (MULTI)POLYGON")
   }
 
   command <- make_lines_call(fields)
 
-  ms_sf(input, command)
+  ms_sf(input, command, sys = sys)
 }
 
 make_lines_call <- function(fields) {
